# base for event views

from analyses.analysisbase import forms as app_forms

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from installations.views.main import installations_navigationlist

from utils import crud
from utils import permissions
from utils.forms import create_class_with_meta
from utils.versioning import versionedcrud
from utils.versioning.models import WorkVersion
from utils.versioning.session import get_current_version


class MixIn(object):
    "Common items for object views"
    include_attachments = True
    include_links = True
##    appconfig = app_models.AppConfig
##    model = appconfig.get_model('event')

class display(MixIn, versionedcrud.DisplayObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.EventSheet, model=self.model)

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['currentanalysistype'] = self.appconfig.analysistype
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        breadcrumbs = [{'url': item.scenario.get_absolute_url('causetree'),
            'label': u'%s' % item.scenario} for item in object.get_treeelements() ]
        extra_context['breadcrumbs'] = breadcrumbs

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.EventForm, model=self.model)

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.EventSheet, model=self.model)

    def get_default_url(self, object):
        es = object.get_treeelements()
        if es:
            return es[0].scenario.get_absolute_url('causetree')
        return '/'

class add_to_scenario(MixIn, versionedcrud.AddObjectView):
    form_class = None

    def setup_from_args(self, object_id=None, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.EventForm, model=self.model)
        # setting scenario here explicitly since it is not a parent of the cause
        # but of the treeelement
        # this gets set in the save step
        (self.version, self.specialversion) = get_current_version(self.request)
        self.scenario = versionedcrud.get_versionedobject_or_404(self.appconfig.get_model('scenario'), self.version, v_id = object_id)
        treeparent_id = kwargs.pop('parent_id', None)
        if treeparent_id:
            try:
                self.treeparent = self.appconfig.get_model('treeelement').objects.get(pk=treeparent_id)
            except self.appconfig.get_model('treeelement').DoesNotExist:
                self.treeparent = None
        else:
            self.treeparent = None

    def check_permission(self, object):
        # override because we don't have a real parent defined
        if not self.appconfig.get_model('treeelement').may_add(self.request.user, parent=self.scenario):
            return permissions.redirect_to_not_allowed()

    def save_instance(self, object):
        object.save(self.request.user, parent=self.treeparent, treeelementkwargs={'scenario': self.scenario})
        return object

    def get_default_url(self, object):
        if self.scenario:
            return self.scenario.get_absolute_url('causetree')
        return '/'

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = None
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.EventSheet, model=self.model)
        super(compare, self).setup_from_args(*args, **kwargs)

