# base for suggestion list views

from analyses.analysisbase import forms as app_forms
from analyses.analysisbase import tables as app_tables
from analyses.analysisbase.views.scenario import localnavigationlist
from analyses.analysisbase.views import helpers
from analyses.main import analyses_navigationlist

from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from installations.models import Scope

from utils import crud
from utils import forms as utils_forms
from utils import permissions
from utils.forms import create_class_with_meta
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version
from utils.versioning.models import WorkVersion


class MixIn(object):
    "Common items for object views"
##    appconfig = app_models.AppConfig
##    model = appconfig.get_model('suggestionlist')

    include_attachments = True
    include_links = True

    def add_to_extra_context(self, extra_context, object):
        extra_context['currentanalysistype'] = self.appconfig.analysistype

    tabs = [
        {'name': 'suggestionlist',
            'label': _('Suggestion list')},
        {'name': 'scenarios',
            'label': _('Scenarios')},
    ]

class display_base(MixIn, versionedcrud.DisplayObjectView):
    allow_publish=False

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)

    def add_permissions_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_permissions_to_extra_context(self, extra_context, object)
        if self.allow_publish and object.may_publish(self.request.user):
            extra_context['defaultbuttons'].append({'url': object.get_publish_url(), 'label': _('Publish')})

class display_suggestionlist(display_base):
    form_class = None
    allow_publish=True

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.SuggestionlistSheet, model=self.model)

class display_scenarios(display_base):
    allow_change = False
    allow_delete = False
    form_class = None
    template = 'analysisbase/suggestionlist_scenarios.html'

    def add_to_extra_context(self, extra_context, object):
        super(display_scenarios, self).add_to_extra_context(extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)
        extra_context['scenarios'] = object.scenarios.all()
        buttons = []
        if self.appconfig.get_model('scenario').may_add(self.request.user, suggestionlist=object):
            buttons.append(
                {'url': object.get_add_scenario_url(),
                    'label': _('Add new')})
        extra_context['buttons'] = buttons

##class display(MixIn, crud.TabContainer):
##    tabviews = {
##        'suggestionlist': display_suggestionlist(),
##        'scenarios': display_scenarios(),
##    }
##    default = 'suggestionlist'

class edit(MixIn, versionedcrud.EditObjectView):
    tabs = []
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.SuggestionlistForm, model=self.model)

class delete(MixIn, versionedcrud.DeleteObjectView):
    tabs = []
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.SuggestionlistSheet, model=self.model)

    def get_default_url(self, object):
        return reverse('%s_suggestionlists' % self.appconfig.namespace)

class report(MixIn, versionedcrud.ReportObjectView):
    form_class = utils_forms.ReportForm
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.ReportObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)


class add(MixIn, versionedcrud.AddObjectView):
    tabs = []
    form_class = None

    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.SuggestionlistForm, model=self.model)

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = None
    def setup_from_args(self, *args, **kwargs):
        if not self.form_class:
            self.form_class = create_class_with_meta(app_forms.SuggestionlistSheet, model=self.model)
        super(compare, self).setup_from_args(*args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn.add_to_extra_context(self, extra_context, object)

class publish(MixIn, versionedcrud.PublishObjectView):
    tabs = []

class SuggestionlistListObject(helpers.TableListObject):
    defaultsort = 'name'
    default_table_class = app_tables.SuggestionListsTable

    def get_title(self):
        return u'%s %s' % (_('Suggestion lists'), self.appconfig.analysistype)

class suggestionlists(helpers.TableListView):
    "List of suggestion lists"

    defaultsort = 'name'
    paginate_by = 100
    default_table_class = app_tables.SuggestionListsTable
    allow_pdf = True

    def get_title(self):
        return u'%s %s' % (_('Suggestion lists'), self.appconfig.analysistype)

    def add_to_extra_context(self, extra_context):
        extra_context['navigationlist'] =  analyses_navigationlist(self.appconfig.namespace)
        extra_context['tablist'] =  self.get_navigationlist(self.appconfig.namespace)
        extra_context['currenttab'] = 'suggestionlists'
        if (self.version==WorkVersion) and self.model.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('%s_suggestionlists_add' % self.appconfig.namespace),
                    'label': _('Add'),
                }]

    def get_navigationlist(self, namespace):
        return localnavigationlist(namespace)

class suggestionlists_for_scope(versionedcrud.DisplayObjectView):
    allow_change=False
    allow_delete = False
    table_class = app_tables.SuggestionListsTable

    def __new__(cls, *args, **kwargs):
        cls.model = Scope
        return super(suggestionlists_for_scope, cls).__new__(cls, *args, **kwargs)

    def check_permission(self, object):
        self.queryset = self.appconfig.get_model('suggestionlist').objects.all().get_versionedobjects(version=WorkVersion)
        if self.queryset.count() == 1:
            return HttpResponseRedirect(reverse('%s_scope_suggestionlist' % self.appconfig.namespace, args=[object.v_id, self.queryset[0].v_id]))
# if 0: add a message no SL's available
        c = permissions.check_object_permission(self.request.user, self.appconfig.get_model('scenario'), "add")
        if c:
            return c

    def get_title(self, object):
        return _('Available suggestion lists')

    def add_to_extra_context(self, extra_context, object):
        for item in self.queryset:
            item.url = reverse('%s_scope_suggestionlist' % self.appconfig.namespace, args=[object.v_id, item.v_id])
        table = self.table_class(self.queryset)
        extra_context['table'] = table

class SuggestionListInline(versionedcrud.Inline):
    """
    Inline for SL
    """
    formset_class = None
    title = _('Suggestion list')

    def __new__(cls, *args, **kwargs):
        cls.model = cls.appconfig.get_model('scenario')
        return super(SuggestionListInline, cls).__new__(cls, *args, **kwargs)

    def get_formset_class(self):
        if self.formset_class:
            return self.formset_class
        form = create_class_with_meta(app_forms.SuggestionListScenarioForm, model=self.model)
        formset = type('SuggestionListInlineFormsetChild', (app_forms.SuggestionListInlineFormset,), {'model': self.model, 'form': form})
        return formset

    def setup_formset(self, formset, parent):
        """
        Set the type on the form, so we can regroup in the template
        """
        for form in formset.forms:
            form.type = form.instance.type

    def save_formset(self, formset):
        """
        No need to return anything, just call save on formset
        """
        return formset.save(self.request.user)

class suggestionlist_for_scope(versionedcrud.EditObjectView):
    form_class = None
    tabs = []
    template = 'analysisbase/suggestionlist_for_scope.html'
##    inline = SuggestionListInline

    def __new__(cls, *args, **kwargs):
        cls.model = Scope
        return super(suggestionlist_for_scope, cls).__new__(cls, *args, **kwargs)

    def setup_from_args(self, *args, **kwargs):
        self.suggestionlist = None
        # if we get an SL_id: use it
        # otherwise redirect to SL choice view unless there is only one SL available
        self.SL_id = kwargs.get('SL_id')
        if self.SL_id:
            try:
                self.suggestionlist = self.appconfig.get_model('suggestionlist').objects.get_versionedobject(version=WorkVersion, v_id = self.SL_id)
            except:
                pass
        else:
            SL = self.appconfig.get_model('suggestionlist').objects.all().get_versionedobjects(version=WorkVersion)
            if SL.count() == 1:
                self.suggestionlist = SL[0]

    def check_permission(self, object):
        c = permissions.check_object_permission(self.request.user, self.appconfig.get_model('scenario'), "add")
        if c:
            return c

    def process_request(self):
        self.previewform = None
        self.refform = None
        self.errormessages = []

        if not self.suggestionlist:
            return HttpResponseRedirect(reverse('%s_scope_suggestionlists' % self.appconfig.namespace, args=[self.object.v_id,]))

        inline = self.inline()
        if self.request.method == 'POST':
            formset = inline.get_formset(self.request, self.suggestionlist, target=self.object, queryset=self.suggestionlist.scenarios.all())
            formset.inline = inline
            self.formsets.append(formset)
            if formset.is_valid():
                formset.save(self.request.user)
                self.set_message(self.object)
                return self.get_redirect(self.refform, self.object)
            else:
                self.errormessages = [self.get_errormessage()]
        else:
            formset = inline.get_formset(self.request, self.suggestionlist, target=self.object, queryset=self.suggestionlist.scenarios.all())
            self.formsets.append(formset)

    def get_title(self, object):
        return _('Suggestion list')

    def get_default_url(self, object):
        return object.get_absolute_url('scenarios')


class allsuggestionlistsreport(versionedcrud.ReportNoObjectView):
    """
    Print all available suggestionlists for the analysistype
    Watch out: the model passed in is the suggestionlist model
    """
    form_class = app_forms.AllSuggestionlistsReportForm
    pdfsendmail = True

    def get_form(self):
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, user=self.request.user, appconfig=self.model.appconfig)
        else:
            form = self.form_class(user=self.request.user, appconfig=self.model.appconfig)
        return form

    def set_task(self):
        from analyses.analysisbase.tasks import mail_allsuggestionlists
        if not self.version or (self.version == WorkVersion):
            version_id = None
        else:
            version_id = self.version.pk
        mail_allsuggestionlists.delay(self.model.appconfig, self.request.user, self.language, self.choices, version_id)

    def get_pdf(self):
        from analyses.analysisbase.models import AllSuggestionlists
        return AllSuggestionlists.get_report(self.model.appconfig, self.request.user, language=self.language, choices=self.choices, version=self.version)


