from analyses.analysisbase import tests as base_tests

from analyses.contacts import models as app_models

from django.contrib.auth.models import User

class PermissionsMixIn(object):
    def setPermissions(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_contactscenarios','contacts','contactscenario'),
            ('edit_contactscenarios','contacts','contactscenario'),
            ('display_installations','installations','section'),
            ('display_contactsuggestionlists','contacts','contactsuggestionlist'),
            ('edit_contactsuggestionlists','contacts','contactsuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_contactscenarios','contacts','contactscenario'),
            ('display_installations','installations','section'),
            ('display_contactsuggestionlists','contacts','contactsuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class SettingsMixIn(object):
    sc_model = app_models.ContactScenario
    ev_model = app_models.ContactEvent
    pl_model = app_models.ContactProtectionLayer
    me_model = app_models.ContactMeasure
    plme_model = app_models.ContactProtectionLayer_Measure
    ap_model = app_models.ContactAttentionPoint
    sl_model = app_models.ContactSuggestionlist
    apsl_model = app_models.ContactAPSuggestionlist
    tree_model = app_models.ContactTreeElement
    namespace = "contacts"

class Contacts_SuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.SuggestionlistTest):
    pass

class Contacts_ScenarioTest(PermissionsMixIn, SettingsMixIn, base_tests.ScenarioTest):
    pass

class Contacts_EventTest(PermissionsMixIn, SettingsMixIn, base_tests.EventTest):
    pass

class Contacts_ProtectionLayerTest(PermissionsMixIn, SettingsMixIn, base_tests.ProtectionLayerTest):
    pass

class Contacts_MeasureTest(PermissionsMixIn, SettingsMixIn, base_tests.MeasureTest):
    pass

class Contacts_APSuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.APSuggestionlistTest):
    pass

class Contacts_AttentionPointTest(PermissionsMixIn, SettingsMixIn, base_tests.AttentionPointTest):
    pass

class Contacts_VersioningTest(PermissionsMixIn, SettingsMixIn, base_tests.VersioningTest):
    pass

class Contacts_SearchTestCase(PermissionsMixIn, SettingsMixIn, base_tests.SearchTest):
    searchclasses = {
        'sl_search': app_models.Contacts_SuggestionlistSearch,
        'sc_search': app_models.Contacts_ScenarioSearch,
        'ev_search': app_models.Contacts_EventSearch,
        'pl_search': app_models.Contacts_ProtectionLayerSearch,
        'me_search': app_models.Contacts_MeasureSearch,
        'apsl_search': app_models.Contacts_APSuggestionlistSearch,
        'ap_search': app_models.Contacts_AttentionPointSearch,
        }
