from analyses.analysisbase import tests as base_tests

from analyses.fires import models as app_models

from django.contrib.auth.models import User

class PermissionsMixIn(object):
    def setPermissions(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_firescenarios','fires','firescenario'),
            ('edit_firescenarios','fires','firescenario'),
            ('display_installations','installations','section'),
            ('display_firesuggestionlists','fires','firesuggestionlist'),
            ('edit_firesuggestionlists','fires','firesuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_firescenarios','fires','firescenario'),
            ('display_installations','installations','section'),
            ('display_firesuggestionlists','fires','firesuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class SettingsMixIn(object):
    sc_model = app_models.FireScenario
    ev_model = app_models.FireEvent
    pl_model = app_models.FireProtectionLayer
    me_model = app_models.FireMeasure
    plme_model = app_models.FireProtectionLayer_Measure
    ap_model = app_models.FireAttentionPoint
    sl_model = app_models.FireSuggestionlist
    apsl_model = app_models.FireAPSuggestionlist
    tree_model = app_models.FireTreeElement
    namespace = "fires"


class Fires_SuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.SuggestionlistTest):
    pass

class Fires_ScenarioTest(PermissionsMixIn, SettingsMixIn, base_tests.ScenarioTest):
    pass

class Fires_EventTest(PermissionsMixIn, SettingsMixIn, base_tests.EventTest):
    pass

class Fires_ProtectionLayerTest(PermissionsMixIn, SettingsMixIn, base_tests.ProtectionLayerTest):
    pass

class Fires_MeasureTest(PermissionsMixIn, SettingsMixIn, base_tests.MeasureTest):
    pass

class Fires_APSuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.APSuggestionlistTest):
    pass

class Fires_AttentionPointTest(PermissionsMixIn, SettingsMixIn, base_tests.AttentionPointTest):
    pass

class Fires_VersioningTest(PermissionsMixIn, SettingsMixIn, base_tests.VersioningTest):
    pass

class Fires_SearchTestCase(PermissionsMixIn, SettingsMixIn, base_tests.SearchTest):
    searchclasses = {
        'sl_search': app_models.Fires_SuggestionlistSearch,
        'sc_search': app_models.Fires_ScenarioSearch,
        'ev_search': app_models.Fires_EventSearch,
        'pl_search': app_models.Fires_ProtectionLayerSearch,
        'me_search': app_models.Fires_MeasureSearch,
        'apsl_search': app_models.Fires_APSuggestionlistSearch,
        'ap_search': app_models.Fires_AttentionPointSearch,
        }
