"""
Main views

"""
from analyses.main import analysesmodules
from django import forms
from django.utils.translation import ugettext_lazy as _
from utils import forms as utils_forms
from utils import reports as utils_reports
from utils.versioning import models as versioning_models
from utils.versioning import versionedcrud

class AllSuggestionlistsReportForm(forms.Form):
    def __init__(self, user, *args, **kwargs):
        super(AllSuggestionlistsReportForm, self).__init__(*args, **kwargs)
        fields = utils_forms.treeify(AllSuggestionlists.get_report_formfields(user))
        for k in fields:
            self.fields[k['name']] = k['field']

class allsuggestionlistsreport(versionedcrud.ReportNoObjectView):
    """
    Print all available suggestionlists
    """
    form_class = AllSuggestionlistsReportForm
    pdfsendmail = True

    def get_form(self):
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, user=self.request.user)
        else:
            form = self.form_class(user=self.request.user)
        return form

    def set_task(self):
        from analyses.tasks import mail_allsuggestionlists
        if not self.version or (self.version == versioning_models.WorkVersion):
            version_id = None
        else:
            version_id = self.version.pk
        mail_allsuggestionlists.delay(self.request.user, self.language, self.choices, version_id)

    def get_pdf(self):
        return AllSuggestionlists.get_report(self.request.user, language=self.language, choices=self.choices, version=self.version)

class AllSuggestionlists(object):
    """
    Dummy object for creating reports with all suggestionlists
    """

    @classmethod
    def get_report_formfields(cls, user):
        from analyses.analysisbase.models import AllSuggestionlists
        res = []
        for item in analysesmodules.get_items_sorted():
            res = utils_forms.mix(res, AllSuggestionlists.get_report_formfields(item['appconfig'], user))
        return res

    @classmethod
    def get_report_body(cls, user, language=u'en', choices={}, version=versioning_models.WorkVersion):
        from analyses.analysisbase.models import AllSuggestionlists
        html = []
        for item in analysesmodules.get_items_sorted():
            html.extend(AllSuggestionlists.get_report_body(item['appconfig'], user, language, choices, version))
            html.append('<pdf:nextpage>')
        return html

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        from analyses.analysisbase.models import AllSuggestionlists
        css = []
        for item in analysesmodules.get_items_sorted():
            css.extend(AllSuggestionlists.get_report_css(item['appconfig'], language, choices))
        return css

    @classmethod
    def get_report(cls, user, language=u'en', choices={}, version=versioning_models.WorkVersion):
        body_strings = cls.get_report_body(user, language, choices, version)
        css_files = cls.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)
