# coding: utf-8

from conversion.connector import get_data

from analyses.deviations import models as deviations_models
from analyses.degradations import models as degradations_models
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from django.contrib.auth.models import User

from django.db import transaction
import logging
import datetime
from os import path as os_path
from django.utils.translation import ugettext as _

# TODO: flatten this out: all this nesting is causing way too many queries

# set up some globals:

defaultuser = User.objects.all()[0]

logger = logging.getLogger()

@transaction.commit_on_success
def convert_id2(id2_filename = None):

    if id2_filename:

        logger.debug('converting id2 file %s' % id2_filename)

        # deviation analyses
        convert_deviation_analysis(id2_filename)

        # degradation analyses
        convert_degradation_analysis(id2_filename)

        # Now make a version
        version = versioning_models.Version(name='Import from %s' % os_path.basename(id2_filename))
        version.save(defaultuser)
        logger.debug('End conversion id2 file')

        return 'Converted id2 file %s' % id2_filename


# DEVIATIONS

deviationscenariotypes = {
    (1,1): _('High pressure'),
    (1,3): _('Low pressure'),
    (1,4): _('Other forces than pressure'),
    (2,1): _('Phenomena leading to corrosion or chemical attack'),
    (2,2): _('High temperature (threatening the envelope\'s integrity)'),
    (2,5): _('Phenomena leading to erosion and wear'),
    (2,6): _('Low temperature'),
    (2,11): _('Phenomena leading to cyclic stresses (introducing the risk of metal fatique)'),
    (3,1): _('Manual operations involving the opening of the installation'),
    (3,2): _('Process upsets leading to a release through an opening')
}

def convert_deviation_analysis(id2_filename):

    sql = """
    SELECT COUNT(*) FROM i_kansenbronnen
    WHERE not (maintype=2 and (subtype=1 or subtype=5 or subtype=11));
    """
    results2 = get_data(sql, id2_filename)
    if int(results2[0][0]):
        suggestionlist = deviations_models.DeviationSuggestionlist(
                name = _('Suggestion list deviations converted'),
            )
        suggestionlist.save(defaultuser)
        sql = """
        SELECT kansenbronid,
        kansenbron, beschrijving, maintype, subtype, evaluatie, inwerking, afgewerkt
        FROM i_kansenbronnen
        WHERE not (maintype=2 and (subtype=1 or subtype=5 or subtype=11));
        """

        results2 = get_data(sql, id2_filename)
        for item in results2:
            convert_deviationscenario(item, suggestionlist, id2_filename)

def convert_deviationscenario(dbeventsource, suggestionlist, id2_filename):
    sql = """
    SELECT B.vrijzettingid,
    vrijzetting, beschrijving, target
    FROM i_kansenbronnen_vrijzettingen A inner join i_vrijzettingen B on A.vrijzettingid=B.vrijzettingid
    WHERE A.kansenbronid=%s;
    """ % dbeventsource[0]

    results = get_data(sql, id2_filename)
    for item in results:
        create_deviationscenario(dbeventsource, item, suggestionlist, id2_filename)
    if not results:
        create_deviationscenario(dbeventsource, None, suggestionlist, id2_filename)

def create_deviationscenario(dbeventsource, dbloc, suggestionlist, id2_filename):

    if dbloc:
        tf = dbloc[3]
    else:
        tf = 0
    # create the event source
    deviationscenario = deviations_models.DeviationScenario(
            suggestionlist = suggestionlist,
            name = dbeventsource[1] and dbeventsource[1][:100],
            type = deviationscenariotypes.get((dbeventsource[3],dbeventsource[4])),
            description = dbeventsource[2],
            evaluation = dbeventsource[5],
            use_LOPA = dbeventsource[7],
            target_frequency = tf,
        )
    deviationscenario.save(defaultuser)

    if dbloc:
        # start causetree with loc
        loc = deviations_models.Cause(
            name = dbloc[1] and dbloc[1][:100],
            description = dbloc[2],
        )
        parent = loc.save(defaultuser, parent=None, treeelementkwargs={'scenario':deviationscenario})

        # now do the measures
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM i_effecten_maatregelen A inner join i_maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and vrijzettingid=%s
        ORDER BY volgorde desc;
        """ % (dbeventsource[0], dbloc[0])
        results = get_data(sql, id2_filename)
        for item in results:
            parent = convert_measure(item, deviationscenario, parent, id2_filename)
    else:
        parent = None

    # insert effect in tree
    cause = deviations_models.Cause(
        name = dbeventsource[6] and dbeventsource[6][:100],
        description = None,
        frequency = None,
    )
    parent = cause.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})

    # insert deviationscenario in tree
    cause = deviations_models.Cause(
        name = dbeventsource[1] and dbeventsource[1][:100],
        description = None,
        frequency = None,
    )
    parent = cause.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})

    # rest of cause tree: recursively
    convert_kansenbronboom(dbeventsource[0], 0, deviationscenario, parent, id2_filename)

    logger.debug('added deviation event source %s' % dbeventsource[1])


def convert_measure(item, deviationscenario, parent, id2_filename):
    # No generation of a implementation, only PL
    # reload from database to have correct mptt info
    if parent:
        parent = deviations_models.CauseTreeElement.objects.get(pk=parent.pk)
    t = item[2]
    if item[3]:
        if t:
            t = t + '\n\nDependencies:\n' + item[3]
        else:
            t = 'Dependencies:\n' + item[3]

    instance = deviations_models.ProtectionLayer(
        name = item[1] and item[1][:100],
        description = t,
        effectivity = item[5],
        reliability = item[4],
        SL = True,
    )
    if item[6] == 1:
        instance.reltype = 1
        instance.rel = item[7]
    elif item[6] == 2:
        instance.reltype = 2
        instance.rel = item[7]
    elif item[6] == 4:
        instance.reltype = 3
        instance.rel = item[7] and (1 - item[7])
    treeelement = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})

    return treeelement

def convert_kansenbronboom(kansenbronid, boven, deviationscenario, parent, id2_filename):
    sql = """
    SELECT oorzaakid,
    oorzaak, beschrijving, condition, freq, boven, poort
    FROM i_oorzaken
    WHERE kansenbronid=%s and boven=%s;
    """ % (kansenbronid, boven)

    results = get_data(sql, id2_filename)
    firstitem = True
    for item in results:
        oldparent = parent
        # reload from database to have correct mptt info
        if parent:
            parent = deviations_models.CauseTreeElement.objects.get(pk=parent.pk)

        if firstitem:
            firstitem = False
            if item[6] == 2:
                parent.gate = True
                parent.save(defaultuser)

        # first convert maatregelen
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM i_oorzaken_maatregelen A inner join i_maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and oorzaakid=%s
        ORDER BY volgorde desc;
        """ % (kansenbronid, item[0])
        results2 = get_data(sql, id2_filename)
        for item2 in results2:
            parent = convert_measure(item2, deviationscenario, parent, id2_filename)
        if item[3]:
            instance = deviations_models.Cause(
                name = item[1] and item[1][:100],
                condition = True,
                description = item[2],
                probability = item[4],
            )
            parent = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})
        else:
            instance = deviations_models.Cause(
                name = item[1] and item[1][:100],
                description = item[2],
                frequency = item[4],
            )
            parent = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})
        convert_kansenbronboom(kansenbronid, item[0], deviationscenario, parent, id2_filename)
        parent = oldparent



# DEGRADATIONS

def convert_degradation_analysis(id2_filename):
    sql = """
    SELECT COUNT(*) FROM i_kansenbronnen
    WHERE maintype=2 and (subtype=1 or subtype=5 or subtype=11);
    """
    results2 = get_data(sql, id2_filename)
    if int(results2[0][0]):
        suggestionlist = degradations_models.DegradationSuggestionlist(
                name = _('Suggestion list degradations converted'),
            )
        suggestionlist.save(defaultuser)
        sql = """
        SELECT kansenbronid,
        kansenbron, beschrijving, maintype, subtype, evaluatie, inwerking, afgewerkt
        FROM i_kansenbronnen
        WHERE maintype=2 and (subtype=1 or subtype=5 or subtype=11);
        """

        results2 = get_data(sql, id2_filename)
        for item in results2:
            convert_degradationscenario(item, suggestionlist, id2_filename)

def convert_degradationscenario(dbeventsource, suggestionlist, id2_filename):
    sql = """
    SELECT B.vrijzettingid,
    vrijzetting, beschrijving, target
    FROM i_kansenbronnen_vrijzettingen A inner join i_vrijzettingen B on A.vrijzettingid=B.vrijzettingid
    WHERE A.kansenbronid=%s;
    """ % dbeventsource[0]

    results = get_data(sql, id2_filename)
    for item in results:
        create_degradationscenario(dbeventsource, item, suggestionlist, id2_filename)
    if not results:
        create_degradationscenario(dbeventsource, None, suggestionlist, id2_filename)


def create_degradationscenario(dbeventsource, dbloc, suggestionlist, id2_filename):

    # create the scenario
    degradationscenario = degradations_models.DegradationScenario(
            suggestionlist = suggestionlist,
            name = dbeventsource[1] and dbeventsource[1][:100],
            type = deviationscenariotypes.get((dbeventsource[3],dbeventsource[4])),
            description = dbeventsource[2],
            evaluation = dbeventsource[5],
        )
    degradationscenario.save(defaultuser)

    if dbloc:
        # start causetree with loc
        event = degradations_models.DegradationEvent(
            name = dbloc[1] and dbloc[1][:100],
            description = dbloc[2],
        )
        parent = event.save(defaultuser, parent=None, treeelementkwargs={'scenario':degradationscenario})

        # now do the measures
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM i_effecten_maatregelen A inner join i_maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and vrijzettingid=%s
        ORDER BY volgorde desc;
        """ % (dbeventsource[0], dbloc[0])
        results = get_data(sql, id2_filename)
        for item in results:
            parent = convert_degradationprotectionlayer(item, degradationscenario, parent, id2_filename)
    else:
        parent = None

    # insert effect in tree
    event = degradations_models.DegradationEvent(
        name = dbeventsource[6] and dbeventsource[6][:100],
        description = None,
    )
    parent = event.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})

    # insert eventsource in tree
    event = degradations_models.DegradationEvent(
        name = dbeventsource[1] and dbeventsource[1][:100],
        description = None,
    )
    parent = event.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})

    # rest of cause tree: recursively
    convert_degradationtree(dbeventsource[0], 0, degradationscenario, parent, id2_filename)

    logger.debug('added degradation event source %s' % dbeventsource[1])

def convert_degradationprotectionlayer(item, degradationscenario, parent, id2_filename):
    # reload from database to have correct mptt info
    if parent:
        parent = degradations_models.DegradationTreeElement.objects.get(pk=parent.pk)

    instance = degradations_models.DegradationProtectionLayer(
        name = item[1] and item[1][:100],
        description = item[2],
        SL = True,
    )
    treeelement = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})

    return treeelement

def convert_degradationtree(kansenbronid, boven, degradationscenario, parent, id2_filename):
    sql = """
    SELECT oorzaakid,
    oorzaak, beschrijving, condition, freq, boven, poort
    FROM i_oorzaken
    WHERE kansenbronid=%s and boven=%s;
    """ % (kansenbronid, boven)

    results = get_data(sql, id2_filename)
    firstitem = True
    for item in results:
        oldparent = parent
        # reload from database to have correct mptt info
        if parent:
            parent = degradations_models.DegradationTreeElement.objects.get(pk=parent.pk)

        if firstitem:
            firstitem = False
            if item[6] == 2:
                parent.gate = True
                parent.save(defaultuser)

        # first convert maatregelen
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM i_oorzaken_maatregelen A inner join i_maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and oorzaakid=%s
        ORDER BY volgorde desc;
        """ % (kansenbronid, item[0])
        results2 = get_data(sql, id2_filename)
        for item2 in results2:
            parent = convert_degradationprotectionlayer(item2, degradationscenario, parent, id2_filename)
        instance = degradations_models.DegradationEvent(
            name = item[1] and item[1][:100],
            description = item[2],
        )
        parent = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})
        convert_degradationtree(kansenbronid, item[0], degradationscenario, parent, id2_filename)
        parent = oldparent


