# coding: utf-8

from conversion.connector import get_data

from installations import models as installations_models
from hazards import models as hazards_models
from analyses.deviations import models as deviations_models
from analyses.degradations import models as degradations_models
from tasks import models as tasks_models
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from django.contrib.auth.models import User

from django.db import transaction
import logging
import datetime
from os import path as os_path
from django.utils.translation import ugettext as _

# TODO: link and OLE fields

# TODO: flatten this out: all this nesting is causing way too many queries

# set up some globals:

mtypedict = {}
installationdict = {}

deviationsmeasuredict = {}
deviationsmeasureelementdict = {}
deviationscauseelementdict = {}

degradationsmeasuredict = {}

defaultuser = User.objects.all()[0]

logger = logging.getLogger()

@transaction.commit_on_success
def convert_pd2(pd2_filename = None, sd2_filename=None, id2_filename=None):

    if pd2_filename and sd2_filename and id2_filename:

        logger.debug('converting pd2 file %s' % pd2_filename)

        # preload measuretypes
        sql = """
        SELECT mtypeid, mtype from mtype;
        """
        results = get_data(sql, id2_filename)
        for item in results:
            mtypedict[item[0]] = item[1] and item[1][:100]

        # installations

        sql = """
        SELECT installatieid,
        installatienaam, beschrijving
        FROM Installaties ORDER BY volgorde;
        """
        results = get_data(sql, pd2_filename)
        counter = 0
        firstinst = None
        for row in results:
            treeelement = create_section(None, row[1], row[2], defaultuser)
            installationdict[row[0]] = treeelement
            counter += 1
            if not firstinst:
                firstinst = treeelement.section

        # convert unlinked tasks into the first installation
        if firstinst:
            convert_unlinked_tasks(firstinst, pd2_filename)

        # sections
        sql = """
        SELECT sectieid,
        sectienaam, beschrijving, installatieid
        FROM secties
        WHERE boven=0 ORDER BY volgorde;
        """
        results = get_data(sql, pd2_filename)
        for row in results:
            counter += convert_section(row, None, pd2_filename, sd2_filename)
        logger.debug('added %s sections' % counter)


        # measure dependencies
        convert_deviations_dep_maatregelen(pd2_filename)
        convert_deviations_dep_oorzaken(pd2_filename)

        # tasks for substances and reactions
        convert_tasks_for_substances(pd2_filename, sd2_filename)
        convert_tasks_for_reactions(pd2_filename, sd2_filename)


        # Now make a version
        version = versioning_models.Version(name='Import from %s' % os_path.basename(pd2_filename))
        version.save(defaultuser)
        logger.debug('End conversion pd2 file')

        return 'Converted pd2 file %s' % pd2_filename

def convert_section(row, parent, pd2_filename, sd2_filename):
    if not parent:
        parent = installationdict.get(row[3])
    instance_el = create_section(parent, row[1], row[2], defaultuser)
    counter = 1
    sql = """
    SELECT sectieid,
    sectienaam, beschrijving, installatieid
    FROM secties
    WHERE boven=%s ORDER BY volgorde;
    """ % row[0]
    results = get_data(sql, pd2_filename)
    for child in results:
        counter += convert_section(child, instance_el, pd2_filename, sd2_filename)

    sql = """
    SELECT onderdeelid,
    onderdeelnaam, beschrijving
    FROM onderdelen
    WHERE sectieid=%s ORDER BY volgorde;
    """ % row[0]
    results = get_data(sql, pd2_filename)
    for child in results:
        cinstance_el = create_section(instance_el, child[1], child[2], defaultuser)
        cinstance = cinstance_el.section
        counter += 1

        # get the auto created scope
        scope = installations_models.Scope.objects.filter(name=child[1]).order_by('-id')[0]

        # substances
        sql = """
        SELECT onderdeelid,
        stofid, aggregatie, hoeveelheidgewenst, hoeveelheidongewenst, toelichting
        FROM onderdelen_stoffen
        WHERE onderdeelid=%s;
        """ % child[0]

        results2 = get_data(sql, pd2_filename)
        for item in results2:
            sql2 = "SELECT stofnaam FROM stoffen WHERE stofid=%s" % item[1]
            s = get_data(sql2, sd2_filename)
            if s:
                substancename = s[0][0]
                try:
                    substance = hazards_models.Substance.objects.get(name=substancename, version__isnull=True)
                except:
                    logger.error('Error: substance not found: %s' % substancename)
                    substance=None
                if substance:
                    cinstance2 = installations_models.Section_Substance(
                        section = cinstance,
                        substance = substance,
                        aggregation = item[2] and item[2][:30],
                        quantity_desired = item[3] and item[3][:50],
                        quantity_undesired = item[4] and item[4][:50],
                        description = item[5],
                        )
                    cinstance2.save(defaultuser)
            else:
                logger.error('Error: substance not found: stofid=%s' % item[1])

        # reactions
        sql = """
        SELECT onderdeelid,
        reactieid, gewenst, beschrijving
        FROM onderdelen_reacties
        WHERE onderdeelid=%s;
        """ % child[0]

        results2 = get_data(sql, pd2_filename)
        for item in results2:
            sql2 = "SELECT reactienaam FROM reacties WHERE reactieid=%s" % item[1]
            s = get_data(sql2, sd2_filename)
            if s:
                reactionname = s[0][0]
                try:
                    reaction = hazards_models.Reaction.objects.get(name=reactionname, version__isnull=True)
                except:
                    logger.error('Error: reaction not found: %s' % reactionname)
                    reaction=None
                if reaction:
                    cinstance2 = installations_models.Section_Reaction(
                        section = cinstance,
                        reaction = reaction,
                        desired = item[2],
                        description = item[3],
                        )
                    cinstance2.save(defaultuser)
            else:
                logger.error('Error: reaction not found: reactieid=%s' % item[1])

##        # components
##        sql = """
##        SELECT onderdeelid,
##        componentid, componentnaam, code
##        FROM componenten
##        WHERE onderdeelid=%s;
##        """ % child[0]
##
##        results2 = get_data(sql, pd2_filename)
##        for item in results2:
##            treeelement = create_section(cinstance_el,item[2], item[3], defaultuser)
##            cinstance2 = treeelement.section

        # weak points
        sql = """
        SELECT onderdeelid,
        zwakpuntid, zwakpunt
        FROM zwakkepunten
        WHERE onderdeelid=%s;
        """ % child[0]

        results2 = get_data(sql, pd2_filename)
        for item in results2:
            cinstance2 = installations_models.Weakpoint(
                section = cinstance,
                name = item[2] and item[2][:100],
                )
            cinstance2.save(defaultuser)

        # deviation analyses
        convert_deviation_analysis(child, scope, pd2_filename)

        # degradation analyses
        convert_degradation_analysis(child, scope, pd2_filename)

        # tasks
        convert_tasks(cinstance, 1, child[0], pd2_filename)

        logger.debug('added subsystem %s' % child[1])

    return counter

def create_section(parent, name, description, defaultuser):
    section = installations_models.Section(
        name = name and name[:100],
        description = description,
        )
    # reload from database to have correct mptt info
    if parent:
        parent = installations_models.SectionTreeElement.objects.get(pk=parent.pk)
    treeelement = section.save(defaultuser, parent=parent)
    return treeelement

# DEVIATIONS

deviationscenariotypes = {
    (1,1): _('High pressure'),
    (1,3): _('Low pressure'),
    (1,4): _('Other forces than pressure'),
    (2,1): _('Phenomena leading to corrosion or chemical attack'),
    (2,2): _('High temperature (threatening the envelope\'s integrity)'),
    (2,5): _('Phenomena leading to erosion and wear'),
    (2,6): _('Low temperature'),
    (2,11): _('Phenomena leading to cyclic stresses (introducing the risk of metal fatique)'),
    (3,1): _('Manual operations involving the opening of the installation'),
    (3,2): _('Process upsets leading to a release through an opening')
}

def convert_deviation_analysis(dbsection, scope, pd2_filename):

    sql = """
    SELECT kansenbronid,
    kansenbron, beschrijving, maintype, subtype, evaluatie, inwerking, afgewerkt
    FROM kansenbronnen
    WHERE onderdeelid=%s and not (maintype=2 and (subtype=1 or subtype=5 or subtype=11));
    """ % dbsection[0]

    results2 = get_data(sql, pd2_filename)
    for item in results2:
        convert_deviationscenario(item, scope, pd2_filename)

def convert_deviationscenario(dbeventsource, scope, pd2_filename):
    sql = """
    SELECT B.vrijzettingid,
    vrijzetting, beschrijving, target
    FROM kansenbronnen_vrijzettingen A inner join vrijzettingen B on A.vrijzettingid=B.vrijzettingid
    WHERE A.kansenbronid=%s;
    """ % dbeventsource[0]

    results = get_data(sql, pd2_filename)
    ds = None
    if not results:
        ds = create_deviationscenario(dbeventsource, None, scope, pd2_filename)
    else:
        if len(results) > 1:
            add_loc = True
        else:
            add_loc = False
        for item in results:
            ds = create_deviationscenario(dbeventsource, item, scope, pd2_filename, add_loc)
            # tasks from vrijzetting
            convert_tasks(ds, 3, item[0], pd2_filename)
    if ds:
        # tasks from kansenbron
        # add to last scenario if multiple
        convert_tasks(ds, 2, dbeventsource[0], pd2_filename)

def create_deviationscenario(dbeventsource, dbloc, scope, pd2_filename, add_loc=False):

    # create the event source
    if add_loc and dbloc:
        name = u'%s - %s' % (dbeventsource[1], dbloc[1])
    else:
        name = dbeventsource[1]
    name = name[:100]
    if dbloc:
        tf = dbloc[3]
    else:
        tf = 0

    deviationscenario = deviations_models.DeviationScenario(
            name = name,
            type = deviationscenariotypes.get((dbeventsource[3],dbeventsource[4])),
            description = dbeventsource[2],
            evaluation = dbeventsource[5],
            use_LOPA = dbeventsource[7],
            target_frequency = tf,
        )
    deviationscenario.scope = scope
    deviationscenario.save(defaultuser)

    if dbloc:
        # start causetree with loc
        loc = deviations_models.Cause(
            name=dbloc[1] and dbloc[1][:100],
            description = dbloc[2],
        )
        parent = loc.save(defaultuser, parent=None, treeelementkwargs={'scenario':deviationscenario})

        # now do the measures
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM effecten_maatregelen A inner join maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and vrijzettingid=%s
        ORDER BY volgorde desc;
        """ % (dbeventsource[0], dbloc[0])
        results = get_data(sql, pd2_filename)
        for item in results:
            parent = convert_measure(item, deviationscenario, parent, pd2_filename)
    else:
        parent = None

    # insert effect in tree
    cause = deviations_models.Cause(
        name = dbeventsource[6] and dbeventsource[6][:100] or u'X',
        description = None,
        frequency = None,
    )
    parent = cause.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})

    # now do the measures
    sql = """
    SELECT B.maatregelid,
    B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
    B.highdemand, B.PFD, B.mtypeid
    FROM kansenbronnen_maatregelen A inner join maatregelen B on A.maatregelid=B.maatregelid
    WHERE kansenbronid=%s
    ORDER BY volgorde desc;
    """ % dbeventsource[0]
    results = get_data(sql, pd2_filename)
    for item in results:
        parent = convert_measure(item, deviationscenario, parent, pd2_filename)

    # insert deviationscenario in tree
    cause = deviations_models.Cause(
        name = dbeventsource[1] and dbeventsource[1][:100],
        description = None,
        frequency = None,
    )
    parent = cause.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})

    # rest of cause tree: recursively
    convert_kansenbronboom(dbeventsource[0], 0, deviationscenario, parent, pd2_filename)

    logger.debug('added deviation event source %s' % dbeventsource[1])

    return deviationscenario

def convert_measure(item, deviationscenario, parent, pd2_filename):
    # reload from database to have correct mptt info
    if parent:
        parent = deviations_models.CauseTreeElement.objects.get(pk=parent.pk)
    t = item[2]
    if item[3]:
        if t:
            t = t + '\n\nDependencies:\n' + item[3]
        else:
            t = 'Dependencies:\n' + item[3]

    m = deviationsmeasuredict.get(item[0])
    if not m:
        instance = deviations_models.DeviationMeasure(
            name = item[1] and item[1][:100],
            description = t,
            type = mtypedict.get(item[8])
        )
        instance.save(defaultuser)
        deviationsmeasuredict[item[0]] = instance

        # tasks from maatregel
        convert_tasks(instance, 4, item[0], pd2_filename)

        # convert aandachtspunten
        sql = """
        SELECT aandachtid,
        aandachtspunt, toelichting, aandachttype
        FROM Aandachtspunten
        WHERE maatregelid=%s
        ORDER BY volgorde;
        """ % item[0]
        results3 = get_data(sql, pd2_filename)
        for item3 in results3:
            instance3 = deviations_models.AttentionPoint(
                measure = instance,
                name = item3[1] and item3[1][:100],
                description = item3[2],
                type = item3[3] and item3[3][:100]
            )
            instance3.save(defaultuser)

        m = instance

    instance = deviations_models.ProtectionLayer(
        name = item[1] and item[1][:100],
        description = t,
        effectivity = item[5],
        reliability = item[4],
    )
    if item[6] == 1:
        instance.reltype = 1
        instance.rel = item[7]
    elif item[6] == 2:
        instance.reltype = 2
        instance.rel = item[7]
    elif item[6] == 4:
        instance.reltype = 3
        instance.rel = item[7] and (1 - item[7])
    treeelement = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})
    deviations_models.DeviationProtectionLayer_Measure(protectionlayer=instance, measure=m).save(defaultuser)
    deviationsmeasureelementdict.setdefault(item[0], []).append(treeelement)

    return treeelement

def convert_kansenbronboom(kansenbronid, boven, deviationscenario, parent, pd2_filename):
    sql = """
    SELECT oorzaakid,
    oorzaak, beschrijving, condition, freq, boven, poort, manueel
    FROM oorzaken
    WHERE kansenbronid=%s and boven=%s
    ORDER BY volgorde;
    """ % (kansenbronid, boven)

    results = get_data(sql, pd2_filename)
    firstitem = True
    for item in results:
        oldparent = parent
        # reload from database to have correct mptt info
        if parent:
            parent = deviations_models.CauseTreeElement.objects.get(pk=parent.pk)

        if firstitem:
            firstitem = False
            if item[6] == 2:
                parent.gate = True
                parent.save(defaultuser)

        # first convert maatregelen
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM oorzaken_maatregelen A inner join maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and oorzaakid=%s
        ORDER BY volgorde desc;
        """ % (kansenbronid, item[0])
        results2 = get_data(sql, pd2_filename)
        for item2 in results2:
            parent = convert_measure(item2, deviationscenario, parent, pd2_filename)

        instance = deviations_models.Cause(
            name = item[1] and item[1][:100],
            description = item[2]
        )
        if item[3]:
            instance.condition = True
            if item[7]:
                instance.probability = item[4]
        else:
            if item[7]:
                instance.frequency = item[4]
        parent = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':deviationscenario})
        deviationscauseelementdict.setdefault((kansenbronid, item[0]), []).append(parent)
        convert_kansenbronboom(kansenbronid, item[0], deviationscenario, parent, pd2_filename)
        parent = oldparent


def convert_deviations_dep_maatregelen(pd2_filename):
    # convert dependencies
    sql = """
    SELECT maatregelid,
    maatregelid2
    FROM dep_maatregelen;
    """
    results = get_data(sql, pd2_filename)
    counter = 0
    for item in results:
        m = deviationsmeasureelementdict.get(item[0])
        o = deviationsmeasureelementdict.get(item[1])
        if m and o:
            for x in m:
                for y in o:
                    if x.tree_id == y.tree_id:
                        instance = deviations_models.Dependency(
                            element1 = x,
                            element2 = y
                        )
                        instance.save(defaultuser)
                        counter += 1
    logger.debug('added %s protection layer interdependencies' % counter)

def convert_deviations_dep_oorzaken(pd2_filename):
    # convert dependencies
    sql = """
    SELECT maatregelid,
    kansenbronid, oorzaakid
    FROM dep_oorzaken;
    """
    results = get_data(sql, pd2_filename)
    counter = 0
    for item in results:
        m = deviationsmeasureelementdict.get(item[0])
        o = deviationscauseelementdict.get((item[1], item[2]))
        if m and o:
            for x in m:
                for y in o:
                    if x.tree_id == y.tree_id:
                        instance = deviations_models.Dependency(
                            element1 = x,
                            element2 = y
                        )
                        instance.save(defaultuser)
                        counter += 1
    logger.debug('added %s protection layer - cause dependencies' % counter)

# DEGRADATIONS

def convert_degradation_analysis(dbsection, scope, pd2_filename):
    sql = """
    SELECT kansenbronid,
    kansenbron, beschrijving, maintype, subtype, evaluatie, inwerking, afgewerkt
    FROM kansenbronnen
    WHERE onderdeelid=%s and maintype=2 and (subtype=1 or subtype=5 or subtype=11);
    """ % dbsection[0]

    results2 = get_data(sql, pd2_filename)
    for item in results2:
        convert_degradationscenario(item, scope, pd2_filename)

def convert_degradationscenario(dbeventsource, scope, pd2_filename):
    sql = """
    SELECT B.vrijzettingid,
    vrijzetting, beschrijving, target
    FROM kansenbronnen_vrijzettingen A inner join vrijzettingen B on A.vrijzettingid=B.vrijzettingid
    WHERE A.kansenbronid=%s;
    """ % dbeventsource[0]

    results = get_data(sql, pd2_filename)
    degradationscenario = None
    if not results:
        degradationscenario = create_degradationscenario(dbeventsource, None, scope, pd2_filename)
    else:
        if len(results) > 1:
            add_loc = True
        else:
            add_loc = False
        for item in results:
            degradationscenario = create_degradationscenario(dbeventsource, item, scope, pd2_filename, add_loc)
            # tasks from vrijzetting
            convert_tasks(degradationscenario, 3, item[0], pd2_filename)
    if degradationscenario:
        # tasks
        # add to last scenario if multiple
        convert_tasks(degradationscenario, 2, dbeventsource[0], pd2_filename)


def create_degradationscenario(dbeventsource, dbloc, scope, pd2_filename, add_loc=False):

    # create the event source
    if add_loc and dbloc:
        name = u'%s - %s' % (dbeventsource[1], dbloc[1])[:100]
    else:
        name = dbeventsource[1] and dbeventsource[1][:100]

    # create the scenario
    degradationscenario = degradations_models.DegradationScenario(
            name = name,
            type = deviationscenariotypes.get((dbeventsource[3],dbeventsource[4])),
            description = dbeventsource[2],
            evaluation = dbeventsource[5],
        )
    degradationscenario.scope = scope
    degradationscenario.save(defaultuser)

    if dbloc:
        # start causetree with loc
        event = degradations_models.DegradationEvent(
            name = dbloc[1] and dbloc[1][:100],
            description = dbloc[2],
        )
        parent = event.save(defaultuser, parent=None, treeelementkwargs={'scenario':degradationscenario})

        # now do the measures
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM effecten_maatregelen A inner join maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and vrijzettingid=%s
        ORDER BY volgorde desc;
        """ % (dbeventsource[0], dbloc[0])
        results = get_data(sql, pd2_filename)
        for item in results:
            parent = convert_degradationprotectionlayer(item, degradationscenario, parent, pd2_filename)
    else:
        parent = None

    # insert effect in tree
    event = degradations_models.DegradationEvent(
        name = dbeventsource[6] and dbeventsource[6][:100] or u'X',
        description = None,
    )
    parent = event.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})

    # now do the measures
    sql = """
    SELECT B.maatregelid,
    B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
    B.highdemand, B.PFD, B.mtypeid
    FROM kansenbronnen_maatregelen A inner join maatregelen B on A.maatregelid=B.maatregelid
    WHERE kansenbronid=%s
    ORDER BY volgorde desc;
    """ % dbeventsource[0]
    results = get_data(sql, pd2_filename)
    for item in results:
        parent = convert_degradationprotectionlayer(item, degradationscenario, parent, pd2_filename)

    # insert eventsource in tree
    event = degradations_models.DegradationEvent(
        name = dbeventsource[1] and dbeventsource[1][:100],
        description = None,
    )
    parent = event.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})

    # rest of cause tree: recursively
    convert_degradationtree(dbeventsource[0], 0, degradationscenario, parent, pd2_filename)

    logger.debug('added degradation event source %s' % dbeventsource[1])

    return degradationscenario

def convert_degradationprotectionlayer(item, degradationscenario, parent, pd2_filename):
    # reload from database to have correct mptt info
    if parent:
        parent = degradations_models.DegradationTreeElement.objects.get(pk=parent.pk)
    m = degradationsmeasuredict.get(item[0])
    if not m:
        instance = degradations_models.DegradationMeasure(
            name = item[1] and item[1][:100],
            description = item[2],
            type = mtypedict.get(item[8])
        )
        instance.save(defaultuser)
        degradationsmeasuredict[item[0]] = instance

        # tasks from maatregel
        convert_tasks(instance, 4, item[0], pd2_filename)

        # convert aandachtspunten
        sql = """
        SELECT aandachtid,
        aandachtspunt, toelichting, aandachttype
        FROM Aandachtspunten
        WHERE maatregelid=%s
        ORDER BY volgorde;
        """ % item[0]
        results3 = get_data(sql, pd2_filename)
        for item3 in results3:
            instance3 = degradations_models.DegradationAttentionPoint(
                measure = instance,
                name = item3[1] and item3[1][:100],
                description = item3[2],
                type = item3[3] and item3[3][:100]
            )
            instance3.save(defaultuser)

        m = instance

    instance = degradations_models.DegradationProtectionLayer(
        name = item[1] and item[1][:100],
        description = item[2],
    )
    treeelement = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})
    degradations_models.DegradationProtectionLayer_Measure(protectionlayer=instance, measure=m).save(defaultuser)
    return treeelement

def convert_degradationtree(kansenbronid, boven, degradationscenario, parent, pd2_filename):
    sql = """
    SELECT oorzaakid,
    oorzaak, beschrijving, condition, freq, boven, poort
    FROM oorzaken
    WHERE kansenbronid=%s and boven=%s
    ORDER BY volgorde;
    """ % (kansenbronid, boven)

    results = get_data(sql, pd2_filename)
    firstitem = True
    for item in results:
        oldparent = parent
        # reload from database to have correct mptt info
        if parent:
            parent = degradations_models.DegradationTreeElement.objects.get(pk=parent.pk)

        if firstitem:
            firstitem = False
            if item[6] == 2:
                parent.gate = True
                parent.save(defaultuser)

        # first convert maatregelen
        sql = """
        SELECT B.maatregelid,
        B.maatregel, B.toelichting, B.afhankelijkheden, B.betrouwbaarheid, B.effectiviteit,
        B.highdemand, B.PFD, B.mtypeid
        FROM oorzaken_maatregelen A inner join maatregelen B on A.maatregelid=B.maatregelid
        WHERE kansenbronid=%s and oorzaakid=%s
        ORDER BY volgorde desc;
        """ % (kansenbronid, item[0])
        results2 = get_data(sql, pd2_filename)
        for item2 in results2:
            parent = convert_degradationprotectionlayer(item2, degradationscenario, parent, pd2_filename)
        instance = degradations_models.DegradationEvent(
            name = item[1] and item[1][:100],
            description = item[2],
        )
        parent = instance.save(defaultuser, parent=parent, treeelementkwargs={'scenario':degradationscenario})
        convert_degradationtree(kansenbronid, item[0], degradationscenario, parent, pd2_filename)
        parent = oldparent

def create_task(object, item):
    task = description = u''
    if item[1]:
        if len(item[1]) > 100:
            task = item[1][:100]
            description = item[1]
            if item[5]:
                description += '\nresp: %s' % item[5]
        else:
            task = item[1]
            if item[5]:
                description = 'resp: %s' % item[5]

        instance = tasks_models.Task(
            content_object = object,
            task = task,
            description = description,
            result = item[2],
            duedate = item[3],
            completiondate = item[4],
        )
        instance.save(defaultuser)
        logger.debug('added task %s' % task)

def convert_tasks(object, originetype, origineid, pd2_filename):
    if object:
        sql = """
        SELECT actieid,
        actie, resultaat, streefdatum, uitvoeringsdatum, verantwoordelijke
        FROM Acties
        WHERE originetype=%s and origineid=%s;
        """ % (originetype, origineid)
        results = get_data(sql, pd2_filename)
        for item in results:
            create_task(object, item)

def convert_tasks_for_substances(pd2_filename, sd2_filename):
    sql = """
    SELECT actieid,
    actie, resultaat, streefdatum, uitvoeringsdatum, verantwoordelijke, origineid
    FROM Acties
    WHERE originetype=5;
    """
    results = get_data(sql, pd2_filename)
    for item in results:
        sql2 = "SELECT stofnaam FROM stoffen WHERE stofid=%s" % item[6]
        s = get_data(sql2, sd2_filename)
        if s:
            substancename = s[0][0]
            try:
                substance = hazards_models.Substance.objects.get(name=substancename, version__isnull=True)
            except:
                logger.error('Error: substance not found: %s' % substancename)
                substance=None
            if substance:
                create_task(substance, item)

def convert_tasks_for_reactions(pd2_filename, sd2_filename):
    sql = """
    SELECT actieid,
    actie, resultaat, streefdatum, uitvoeringsdatum, verantwoordelijke, origineid
    FROM Acties
    WHERE originetype=6;
    """
    results = get_data(sql, pd2_filename)
    for item in results:
        sql2 = "SELECT reactienaam FROM reacties WHERE reactieid=%s" % item[6]
        s = get_data(sql2, sd2_filename)
        if s:
            reactionname = s[0][0]
            try:
                reaction = hazards_models.Reaction.objects.get(name=reactionname, version__isnull=True)
            except:
                logger.error('Error: reaction not found: %s' % reactionname)
                reaction=None
            if reaction:
                create_task(reaction, item)

def convert_unlinked_tasks(object, pd2_filename):
    sql = """
    SELECT actieid,
    actie, resultaat, streefdatum, uitvoeringsdatum, verantwoordelijke
    FROM Acties
    WHERE originetype=0;
    """
    results = get_data(sql, pd2_filename)
    for item in results:
        create_task(object, item)
