from django.core.management.base import NoArgsCommand, CommandError
from conversion.convert_pd2 import convert_pd2
from conversion.convert_sd2 import convert_sd2
import logging
import os

try:
    import wx
    wx_available = True
except:
    wx_available = False

if wx_available:
    import wx.lib.filebrowsebutton as filebrowse

    class LogToText(logging.Handler):
      def __init__(self, textCtl):
          logging.Handler.__init__(self)
          self.textCtl = textCtl

      def emit(self, record):
          self.textCtl.AppendText(self.format(record))
          self.textCtl.AppendText('\n')

    class MyFrame(wx.Frame):
        """
        This is MyFrame.  It just shows a few controls on a wxPanel,
        and has a simple menu.
        """
        def __init__(self, parent, title):
            wx.Frame.__init__(self, parent, -1, title,
                              pos=(100, 100), size=(500, 500))
            self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

            # Create the menubar
            menuBar = wx.MenuBar()

            # and a menu
            menu = wx.Menu()

            # add an item to the menu, using \tKeyName automatically
            # creates an accelerator, the third param is some help text
            # that will show up in the statusbar
            menu.Append(wx.ID_EXIT, "E&xit\tAlt-X", "Exit the application")

            # bind the menu event to an event handler
            self.Bind(wx.EVT_MENU, self.OnCloseMe, id=wx.ID_EXIT)

            # and put the menu on the menubar
            menuBar.Append(menu, "&File")
            self.SetMenuBar(menuBar)

            # Now create the Panel to put the other controls on.
            panel = wx.Panel(self)

            # and a few controls
            text = wx.StaticText(panel, -1, "Select the conversion files")
            text.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD))
            text.SetSize(text.GetBestSize())
            btn = wx.Button(panel, -1, "Convert")

            self.status = wx.StaticText(panel, -1, "")
            self.status.SetSize(text.GetBestSize())

            # bind the button events to handlers
            self.Bind(wx.EVT_BUTTON, self.Convert, btn)

            self.fban = filebrowse.FileBrowseButton(
                panel, -1, size=(450, -1), changeCallback = self.fbCallback,
                labelText = 'Analysis file:', fileMask = '*.pd2'
                )

            self.fbsub = filebrowse.FileBrowseButton(
                panel, -1, size=(450, -1), changeCallback = self.fbCallback,
                labelText = 'Substance file:', fileMask = '*.sd2'
                )

            self.fbexp = filebrowse.FileBrowseButton(
                panel, -1, size=(450, -1), changeCallback = self.fbCallback,
                labelText = 'Expert file:', fileMask = '*.id2'
                )

            # Set up a log window
            self.logbox = wx.TextCtrl(panel, -1, size=(450,200),
                                  style = wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)

            sizer = wx.BoxSizer(wx.VERTICAL)
            sizer.Add(text, 0, wx.ALL, 5)
            sizer.Add(self.fban, 0, wx.ALL, 5)
            sizer.Add(self.fbsub, 0, wx.ALL, 5)
            sizer.Add(self.fbexp, 0, wx.ALL, 5)
            sizer.Add(btn, 0, wx.ALL, 5)
            sizer.Add(self.status, 0, wx.ALL, 5)
            sizer.Add(self.logbox, 0, wx.ALL, 5)
            box = wx.BoxSizer()
            box.Add(sizer, 0, wx.ALL, 20)
            panel.SetSizer(box)
            panel.Layout()

        def fbCallback(self, evt):
            obj = evt.GetEventObject()
            if obj:
                current = obj.GetValue()
                directory = os.path.dirname(current)
                for fb in (self.fban, self.fbsub, self.fbexp):
                    if not fb.GetValue():
                        fb.startDirectory = directory

        def OnCloseWindow(self, event):
            self.Destroy()
        def OnCloseMe(self, event):
            self.Close(True)

        def Convert(self, evt):
            """Event handler for the button click."""
            anfile = self.fban.GetValue()
            subfile = self.fbsub.GetValue()
            expfile = self.fbexp.GetValue()
            if not (anfile and subfile and expfile):
                self.status.SetLabel('You must select all files.')
            else:
                self.status.SetLabel('Converting the substance file.')

               # create logger
                logger = logging.getLogger()
                logger.setLevel(logging.DEBUG)
                # create console handler and set level to debug
                ch = LogToText(self.logbox)
                ch.setLevel(logging.DEBUG)
                formatter = logging.Formatter("%(message)s")
                # add formatter to ch
                ch.setFormatter(formatter)
                # add ch to logger
                logger.addHandler(ch)

                convert_sd2(subfile)
                self.status.SetLabel('Converting the analysis file.')
                convert_pd2(anfile, subfile, expfile)
                self.status.SetLabel('Conversion finished')


    class MyApp(wx.App):
        def OnInit(self):
            frame = MyFrame(None, "Planop 2 Converter")
            self.SetTopWindow(frame)
            frame.Show(True)
            return True

class Command(NoArgsCommand):

    def handle_noargs(self, **options):
        if wx_available:
            app = MyApp(False)
            app.MainLoop()
        else:
            print "Error: wx package not available"

