import hazards.models as app_models
from django.utils.translation import ugettext_lazy as _
from utils import tables
from utils.versioning import helpers

class SubstancesTable(tables.ModelTable):
    name = tables.Column(
            verbose_name = _('Name'),
            link = lambda x: x.data.get_absolute_url(),
            sortable=True,
        )
    class Meta:
        model = app_models.Substance
        columns = ('name', 'tag')

class SubstancesCSVTable(tables.ModelTable):
    class Meta:
        model = app_models.Substance
        columns = ('name', 'tag', 'description', 'hazards', 'universal')

class ReactionsTable(helpers.SelectRelatedModelTable):
    name = tables.Column(
            verbose_name = _('Name'),
            link = lambda x: x.data.get_absolute_url(),
            sortable=True,
        )
    substance1 = tables.Column(
            verbose_name = _('Substance 1'),
            link = lambda x: x.data.substance1.get_absolute_url(),
        )
    substance2 = tables.Column(
            verbose_name = _('Substance 2'),
            link = lambda x: x.data.substance2.get_absolute_url(),
        )
    class Meta:
        model = app_models.Reaction
        columns = ('name', 'substance1', 'substance2',)

class ReactionsCSVTable(helpers.SelectRelatedModelTable):
    class Meta:
        model = app_models.Reaction
        columns = ('name', 'substance1', 'substance2', 'description', 'conditions')
