# meeting views

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from meetings import models as app_models
from meetings import forms as app_forms
from meetings import tables as app_tables

from utils import crud
from utils import forms as utils_forms
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version
from utils.versioning.models import WorkVersion

class MixIn(object):
    "Common items for object views"
    model = app_models.Meeting
    include_attachments = True
    include_links = True

class display(MixIn, versionedcrud.DisplayObjectView):
    form_class = app_forms.MeetingSheet

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.MeetingForm

    def save_form(self, form):
        object = form.save(commit=False)
        object = self.save_instance(object)
        # explicit m2m saving (normally not possible with explicit through model)
        for item in form.cleaned_data['participants']:
            app_models.Meeting_User(meeting=object, user=item).save(self.request.user)
        return object

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.MeetingSheet

    def get_default_url(self, object):
        return reverse('meetings_meetings')

class report(MixIn, versionedcrud.ReportObjectView):
    pass

class add(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.MeetingForm

    def save_form(self, form):
        object = form.save(commit=False)
        object = self.save_instance(object)
        # explicit m2m saving (normally not possible with explicit through model)
        for item in form.cleaned_data['participants']:
            app_models.Meeting_User(meeting=object, user=item).save(self.request.user)
        return object

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.MeetingSheet

class MeetingFilter(crud.Filter):
    form = app_forms.MeetingFilterForm
    filters = [
                {'filteritem': 'subject', 'parameter': 'subject__icontains', 'type': 'normal'},
            ]

class MeetingListObject(versionedcrud.TableListObject):
    table_class = app_tables.MeetingsTable
    defaultsort = '-date'
    queryset = app_models.Meeting.objects.all()
    title = _('Meetings')

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('meetings'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        if choices.get('meetings'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class meetings(versionedcrud.TableListView):
    "List of meetings"

    table_class = app_tables.MeetingsTable
    defaultsort = '-date'
    queryset = app_models.Meeting.objects.all()
    paginate_by = 100
    title = _('Meetings')
    filter = MeetingFilter()
##    redirect_on_unique = True
    pdfselectform_class = utils_forms.ListReportForm
    allow_pdf = True
    pdfsendmail = True
    listobject = MeetingListObject()

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.Meeting)

    def add_to_extra_context(self, extra_context):
        super(meetings, self).add_to_extra_context(extra_context)
        if (self.version==WorkVersion) and app_models.Meeting.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('meetings_meeting_add'),
                    'label': _('Add'),
                }]

