# coding=iso-8859-1
# Django settings for planop project.

# thanks to Rob Hudson: http://rob.cogit8.org/blog/2008/Jun/20/django-and-relativity/
from os import path as os_path
PROJECT_PATH = os_path.abspath(os_path.split(__file__)[0])

DEBUG = False
TEMPLATE_DEBUG = DEBUG
SERVE_STATIC = False
USE_CELERY = False

ADMINS = (
     ('Koen Biermans', 'koen.biermans@werk.belgie.be'),
)

MANAGERS = ADMINS

# override internal ips
INTERNAL_IPS = (
    '127.0.0.1',
)

DATABASES = {
    "default": {
        "ENGINE": 'django.db.backends.sqlite3',
        "NAME": os_path.join(PROJECT_PATH, 'db', 'planop.db'),
        "USER": '',
        "PASSWORD": '',
    },
}


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Brussels'

# Is this needed ?
FILE_CHARSET = 'iso-8859-1'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en'

LANGUAGES = (
    ('en', 'English'),
    ('nl', 'Nederlands'),
##    ('fr', u'Franais'),
)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = os_path.join(PROJECT_PATH, 'sitemedia')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/sitemedia/'

# Absolute path to the directory that holds uploads.
# Example: "/srv/uploads/"
UPLOAD_ROOT = os_path.join(PROJECT_PATH, 'uploads')

# URL that handles the uploads served from UPLOAD_ROOT.
# Examples: "/uploads/"
UPLOAD_URL = '/uploads/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '=j02okoiq(bka&2eju-m!^$2azmm(&v3y#xkd5^o@x+kbmsv6#'

# List of callables that know how to import templates from various sources.
# for now let's disable template caching in development:
##TEMPLATE_LOADERS = (
##    ('django.template.loaders.cached.Loader', (
##        'django.template.loaders.filesystem.Loader',
##        'django.template.loaders.app_directories.Loader',
##    )),
##)
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os_path.join(PROJECT_PATH, 'local/templates'),
    os_path.join(PROJECT_PATH, 'templates'),
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
##    'django.contrib.redirects.middleware.RedirectFallbackMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',

)

ROOT_URLCONF = 'urls'

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.request',
    'django.core.context_processors.csrf',
    'django.contrib.auth.context_processors.auth',
    'django.contrib.messages.context_processors.messages',
    'main.context_processors.templates_context_processors.base_templates',
    'main.context_processors.templates_context_processors.navbar',
    'main.context_processors.templates_context_processors.adminmedia',
    'main.context_processors.templates_context_processors.mediaroot',
    'main.context_processors.templates_context_processors.projectname',
    'analyses.analysisbase.context_processors.analyses_context_processors.analysistype',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'django.contrib.markup',
##    'django.contrib.redirects',
##    'django.contrib.admindocs',
    'django.contrib.messages',
    'cpserver',
    'conversion',
    'south',
    'mptt',
    'main',
    'users',
    'utils',
    'utils.exchange',
    'utils.versioning',
    'utils.crud',
    'utils.tables',
    'utils.trees',
    'utils.attachments',
    'utils.links',
    'comments',
    'tasks',
    'utils.news',
    'utils.search',
    'hazards',
    'installations',
    'analyses.analysisbase',
    'analyses.deviations',
    'analyses.degradations',
    'analyses.releases',
    'analyses.dispersions',
    'analyses.fires',
    'analyses.explosionprotection',
    'analyses.contacts',
    'analyses.explosionprevention',
    'meetings',
)

LOGIN_REDIRECT_URL = '/'

SKIP_SOUTH_TESTS = True
##SOUTH_TESTS_MIGRATE = False

CONVERSION_PATH = os_path.join(PROJECT_PATH, 'conversion/data')

DATE_INPUT_FORMATS = (
    '%d/%m/%Y', '%d/%m/%y',  # '25/10/2006', '25/10/06'
    '%d/%m',                 # '25/10'
    '%Y-%m-%d',              # '2006-10-25'
)

DATE_OUTPUT_FORMAT = '%d/%m/%Y'
TIME_OUTPUT_FORMAT = '%H:%M'
DATETIME_OUTPUT_FORMAT = '%s %s' % (DATE_OUTPUT_FORMAT, TIME_OUTPUT_FORMAT)

DATE_FORMAT = 'd/m/Y'
TIME_FORMAT = 'H:i'
DATETIME_FORMAT = '%s %s' % (DATE_FORMAT, TIME_FORMAT)

INSTALLATIONS_OVERVIEW_DEFAULT_LEVEL = 2

HAZARDS_MATRIX_MAX_SUBSTANCES = 25

CELERY_RESULT_BACKEND = "amqp"
BROKER_HOST = "127.0.0.1"
BROKER_PORT = 5672
BROKER_USER = "guest"
BROKER_PASSWORD = "guest"
BROKER_VHOST = "/"
CELERY_IMPORTS = ("analyses.tasks",)

try:
    from local_settings import *
except ImportError:
    pass


if USE_CELERY:
    import djcelery
    djcelery.setup_loader()
    INSTALLED_APPS = INSTALLED_APPS + ('djcelery',)

