"""
Users app

App to manage users and groups from django.contrib.auth

"""

from django.contrib.auth import models as auth_models
from django.db import models
from django.utils.translation import ugettext_lazy as _

class UserManager(models.Manager):
    """
    Custom manager
    """
    def get_query_set(self):
        return super(UserManager, self).get_query_set().exclude(username__startswith='__')

class User(auth_models.User):
    objects = UserManager()
    class Meta:
        proxy = True
        verbose_name = _('User')
        verbose_name_plural = _('Users')
        permissions = (
            ("display_users", "May display users information"),
            ("edit_users", "May edit users information"),
        )

    # urls

    @models.permalink
    def get_absolute_url(self):
        return ('users_user', (self.id,))

    @models.permalink
    def get_edit_url(self):
        return ('users_user_edit', (self.id,))

    @models.permalink
    def get_delete_url(self):
        return ('users_user_delete', (self.id,))

    @models.permalink
    def get_change_password_url(self):
        return ('users_user_set_password', (self.id,))

    # permissions

    @classmethod
    def may_add(cls, user):
        return user.has_perm('auth.edit_users')

    def may_change(self, user):
        return user.has_perm('auth.edit_users')

    def may_delete(self, user):
        return False

class Group(auth_models.Group):
    class Meta:
        proxy = True
        verbose_name = _('Group')
        verbose_name_plural = _('Group')

    # urls

    @models.permalink
    def get_absolute_url(self, tab='group'):
        return ('users_group', (self.id, tab))

    @models.permalink
    def get_edit_url(self, tab='group'):
        return ('users_group_edit', (self.id,))

    @models.permalink
    def get_delete_url(self):
        return ('users_group_delete', (self.id,))

    # permissions

    @classmethod
    def may_add(cls, user):
        return user.has_perm('auth.edit_users')

    def may_change(self, user):
        return user.has_perm('auth.edit_users')

    def may_delete(self, user):
        if user.has_perm('auth.edit_users'):
            return self.user_set.count() == 0
        return False


