# link views

from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.utils.translation import ugettext_lazy as _
from utils.links.models import Link
from utils.links.forms import LinkForm, LinkSheet
from utils import permissions
from utils.versioning import versionedcrud


class delete(versionedcrud.DeleteObjectView):
    model = Link
    form_class = LinkSheet

    def get_default_url(self, object):
        return display_url_for_obj(object.content_object)

class add(versionedcrud.AddObjectView):
    """
    Add a link to an object.
    """
    model = Link
    form_class = LinkForm
    parent_field = 'content_object'

    def __call__(self, request, app_label, module_name, *args, **kwargs):
        self.parent_class = get_model(app_label, module_name)
        return super(add, self).__call__(request, *args, **kwargs)

class display(versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    template = 'links/display.html'
    def __call__(self, request, app_label, module_name, *args, **kwargs):
        self.model = get_model(app_label, module_name)
        return super(display, self).__call__(request, *args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        links = Link.objects.links_for_object(object)
        extra_context['links'] = links
        if object.may_change(self.request.user):
            extra_context['buttons'] = [
                {'url': object.get_absolute_url(),
                    'label': _('Back')},
                {'url': add_url_for_obj(object),
                    'label': _('Add')},
            ]
            extra_context['may_delete'] = True

def add_url_for_obj(obj):
    return reverse('links_add', kwargs={
                        'app_label': obj._meta.app_label,
                        'module_name': obj._meta.module_name,
                        'object_id': obj.v_id
                    })

def display_url_for_obj(obj):
    return reverse('links_display', kwargs={
                        'app_label': obj._meta.app_label,
                        'module_name': obj._meta.module_name,
                        'object_id': obj.v_id
                    })

